package mi.hdm;

import javafx.application.Application;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.stage.Stage;
import mi.hdm.controllers.View;
import mi.hdm.filesystem.FileManager;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;

public class GUIDriver extends Application {
    private final static Logger log = LogManager.getLogger(GUIDriver.class);

    private final static View STARTUP_VIEW = View.MAIN;
    private final static String PATH_TO_ICON = "/images/Tasty_Pages_Icon.png";
    private final static String FILE_NAME = "/music/GrandEtude_DagReinbott.wav";

    private static Stage stage;
    private static TastyPages model;

    public static void main(String[] args) {
        log.info("Starting TastyPages");
        try {
            model = FileManager.deserializeFromFilesystem();
            //model.createRecipe();
        } catch (Exception e) {
            log.fatal("Internal exception, check file paths and URIs");
            e.printStackTrace();
            System.exit(1);
        }

        Thread musicThread = new Thread(){
            public void run() {
                final String ABSOLUTE_FILE_NAME = FileManager.getAbsolutePathFromResourceFolder(FILE_NAME);
                while(!Thread.currentThread().isInterrupted()) {
                    playMusic(ABSOLUTE_FILE_NAME);
                }
            }

        };
        musicThread.setDaemon(true);
        log.info("Start playing music.");
        musicThread.start();

        launch(args);
        log.info("Shutting down application");
        log.info("Saving all data");
        FileManager.serializeToFile(model);
    }

    @Override
    public void start(Stage stage) throws Exception {
        GUIDriver.stage = stage;

        Parent parent = STARTUP_VIEW.getScene();
        final Scene scene = new Scene(parent, 1400, 800);

        stage.setTitle(STARTUP_VIEW.getWindowTitle());
        stage.setMaximized(true); //open scene in full screen
        stage.setScene(scene);
        stage.show();

        InputStream iconStream = getClass().getResourceAsStream(PATH_TO_ICON);
        if (iconStream != null) {
            stage.getIcons().add(new Image(iconStream));
            log.debug("Set icon for application");
        } else log.warn("Could not find icon for application. Check the filepath");
    }

    public static TastyPages getModel() {
        return model;
    }

    public static Stage getApplicationStage() {
        return stage;
    }

    public static View getStartupView() {
        return STARTUP_VIEW;
    }

    private static void playMusic(String fileName) {
        File soundFile = new File(fileName);
        log.debug("Found file {}", soundFile);
        AudioInputStream audioInputStream = null;
        try
        {
            audioInputStream = AudioSystem.getAudioInputStream(soundFile);
            log.info("Found file {}", soundFile);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        AudioFormat audioFormat = audioInputStream.getFormat();
        SourceDataLine line = null;
        DataLine.Info info = new DataLine.Info(SourceDataLine.class, audioFormat);
        try
        {
            line = (SourceDataLine) AudioSystem.getLine(info);
            line.open(audioFormat);
        }
        catch (LineUnavailableException e)
        {
            e.printStackTrace();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        line.start();
        int nBytesRead = 0;
        byte[] abData = new byte[128000];
        while (nBytesRead != -1)
        {
            try
            {
                nBytesRead = audioInputStream.read(abData, 0, abData.length);
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
            if (nBytesRead >= 0)
            {
                int nBytesWritten = line.write(abData, 0, nBytesRead);
            }
        }
        line.drain();
        line.close();
    }
}
