package mi.hdm.controllers;

import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import mi.hdm.GUIDriver;
import mi.hdm.TastyPages;
import mi.hdm.controllers.components.HeaderController;
import mi.hdm.controllers.components.NutritionTableController;
import mi.hdm.recipes.NutritionTable;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;

public enum ViewComponent {
    HEADER("/fxml/components/header.fxml"),
    NUTRITION_TABLE("/fxml/components/nutrition-table.fxml");

    private static final Logger log = LogManager.getLogger(ViewComponent.class);

    private final String path;

    private static final TastyPages model = GUIDriver.getModel();

    ViewComponent(String path) {
        this.path = path;
    }

    public Parent getParentElement(Object... args) throws IOException {
        log.info("Loading FXML for '{}' component from: {}", this, path);
        final FXMLLoader loader = new FXMLLoader(getClass().getResource(path));
        log.debug("Successfully retrieved resource from {}", path);

        switch (this) {
            case HEADER -> loader.setControllerFactory((callback) ->
                    new HeaderController(model.getRecipeManager(), model.getIngredientManager())
            );

            case NUTRITION_TABLE -> {
                if (args[0] instanceof NutritionTable n && args[1] instanceof String text) {
                    loader.setControllerFactory(callback -> new NutritionTableController(n, text));
                } else
                    throw new IOException("Provide arguments to load nutrition table: Nutrition table n, String text");
            }
        }

        log.debug("Set controller factory for view component {}", this);

        return loader.load();
    }
}
