package mi.hdm.helpers;

import java.util.List;
import java.util.stream.Stream;

public class Validation {
    /**
     * Validate Strings before passing them to methods like Integer.parseInt()
     *
     * @param input Input string to be tested
     * @return true if the String is not empty and a valid integer
     */
    public static boolean isInteger(String input) {
        if (input.isBlank()) return false;
        for (int i = 0; i < input.length(); i++) {
            char c = input.charAt(i);
            if (i == 0 && (c == '+' || c == '-')) continue;
            if (!Character.isDigit(input.charAt(i))) return false;
        }
        return true;
    }

    /**
     * @return true if the input string can be parsed by Double.parseDouble
     */
    public static boolean isDouble(String input) {
        if (input.isBlank()) return false;
        boolean hasDot = false;
        for (int i = 0; i < input.length(); i++) {
            char c = input.charAt(i);
            if (i == 0 && (c == '+' || c == '-')) continue;
            if (!hasDot && (c == '.')) {
                hasDot = true;
                continue;
            }
            if (!Character.isDigit(c)) return false;
        }
        return true;
    }

    /**
     * @return a list from the lines of the provided string with no empty lines
     */
    public static List<String> listNoEmptyLines(String input) {
        return Stream.of(input.split("\n")).filter(s -> !s.isBlank()).toList();
    }
}
