package mi.hdm.typeAdapters;

import com.google.gson.Gson;
import com.google.gson.TypeAdapter;
import com.google.gson.reflect.TypeToken;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonWriter;
import mi.hdm.recipes.RecipeManager;
import mi.hdm.shoppingList.ShoppingList;

import java.io.IOException;
import java.util.Map;

public class ShoppingListTypeAdapter extends TypeAdapter<ShoppingList> {
    private final RecipeManager recipeManager;

    public ShoppingListTypeAdapter(RecipeManager recipeManager) {
        this.recipeManager = recipeManager;
    }

    @Override
    public void write(JsonWriter writer, ShoppingList shoppingList) throws IOException {
        final Map<String, Boolean> map = shoppingList.getShoppingListMap();
        writer.beginObject();
        writer.name("shoppingList").jsonValue(new Gson().toJson(map));
        writer.endObject();
    }

    @Override
    public ShoppingList read(JsonReader reader) throws IOException {
        Map<String, Boolean> map = null;
        reader.beginObject();
        String name = reader.nextName();
        if (name.equals("shoppingList")) {
            map = new Gson().fromJson(reader, new TypeToken<Map<String, Boolean>>() {
            }.getType());
        }
        reader.endObject();
        if (map == null) return null;
        return new ShoppingList(map, recipeManager);
    }
}
