package mi.hdm.filesystem;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import mi.hdm.mealPlan.MealPlan;
import mi.hdm.recipes.*;
import mi.hdm.shoppingList.ShoppingList;
import mi.hdm.typeAdapters.*;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.util.List;
import java.util.Map;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class JsonTest {
    private final static IngredientManager ingredientManager = new IngredientManager();
    private final static CategoryManager categoryManager = new CategoryManager();
    private final static RecipeManager recipeManager = new RecipeManager();

    @BeforeAll
    public static void setupAll() {
        ingredientManager.addIngredient(ValidObjectsPool.getValidIngredientTwo());
        ingredientManager.addIngredient(ValidObjectsPool.getValidIngredientOne());

        categoryManager.addCategory("Kategorie 1", 0xFF0000);
        categoryManager.addCategory("Kategorie 2", 0xFF00F8);
    }

    @Test
    public void testCategoryToJSON() {
        Category expected = ValidObjectsPool.getValidCategoryOne();
        Gson gson = new GsonBuilder()
                .registerTypeAdapter(Category.class, new CategoryTypeAdapter())
                .create();

        String jsonResult = gson.toJson(expected);

        Category result = gson.fromJson(jsonResult, Category.class);
        assertEquals(expected, result);
    }

    @Test
    public void testCategoryManagerToJson() {
        Gson gson = new GsonBuilder()
                .registerTypeAdapter(CategoryManager.class, new CategoryManagerTypeAdapter())
                .create();
        String jsonResult = gson.toJson(categoryManager);
        System.out.println(jsonResult);
        CategoryManager result = gson.fromJson(jsonResult, CategoryManager.class);
        assertEquals(categoryManager, result);
    }

    @Test
    public void testNutritionTableToJSON() {
        NutritionTable expected = ValidObjectsPool.getValidNutritionTableOne();
        Gson gson = new GsonBuilder()
                .registerTypeAdapter(NutritionTable.class, new NutritionTableTypeAdapter())
                .create();

        String json = gson.toJson(expected, NutritionTable.class);

        NutritionTable result = gson.fromJson(json, NutritionTable.class);
        assertEquals(expected, result);
    }

    @Test
    public void testRecipeToJSON() {
        Recipe expected = ValidObjectsPool.getValidRecipeOne();
        Gson gson = new GsonBuilder()
                .registerTypeAdapter(Recipe.class, new RecipeTypeAdapter())
                .create();
        String json = gson.toJson(expected);
        System.out.println(json);
        Recipe result = gson.fromJson(json, Recipe.class);
        assertEquals(expected, result);
    }

    @Test
    public void testMealPlanToJson() {
        MealPlan expected = ValidObjectsPool.getValidMealPlanOne();
        Gson gson = new GsonBuilder()
                .registerTypeAdapter(MealPlan.class, new MealPlanTypeAdapter(new RecipeManager()))
                .create();
        String json = gson.toJson(expected);
        System.out.println(json);
        MealPlan result = gson.fromJson(json, MealPlan.class);

        assertEquals(expected, result);
    }

    @Test
    public void testShoppingListToJson() {
        ShoppingList expected = ValidObjectsPool.getShoppingList();
        Gson gson = new GsonBuilder()
                .registerTypeAdapter(ShoppingList.class, new ShoppingListTypeAdapter(recipeManager))
                .create();
        String json = gson.toJson(expected);
        System.out.println(json);
        ShoppingList result = gson.fromJson(json, ShoppingList.class);

        assertEquals(expected, result);
    }

    @Test
    public void getCorrectObjectsFromRecipe() {
        Ingredient ingredient = ValidObjectsPool.getValidIngredientOne();
        Category category = ValidObjectsPool.getValidCategoryOne();
        Recipe recipe = new Recipe("Expect", Map.of(ingredient, 100), "Description", List.of("Prepare"), List.of(category), 15);

        IngredientManager ingredientManager = new IngredientManager(List.of(ingredient));
        CategoryManager categoryManager = new CategoryManager(List.of(category));

        assertEquals(category, categoryManager.get(category.getCode()).orElseThrow());
        assertEquals(ingredient, ingredientManager.getIngredient(ingredient.getUniqueCode()).orElseThrow());
    }
}
