package mi.hdm.recipes;

import mi.hdm.mealPlan.MealPlan;
import mi.hdm.shoppingList.ShoppingList;

import java.time.LocalDate;
import java.util.List;
import java.util.Map;

public class ValidObjectsPool {
    private final static RecipeManager recipeManager = new RecipeManager();
    private final static Category categoryOne = new Category("Frühstück", 0xFF0000);
    private final static Category categoryTwo = new Category("Abendessen", 0x00FF00);
    private final static NutritionTable nutritionTableOne = new NutritionTable(100.0, 17.2, 9.4, 4.3, 2.4, 0.4);
    private final static NutritionTable nutritionTableTwo = new NutritionTable(263.1, 25.2, 0.2, 0.0, 0.0, 0.8);
    private final static Ingredient ingredientOne = new Ingredient(Measurement.GRAM, "Zucker", nutritionTableOne);
    private final static Ingredient ingredientTwo = new Ingredient(Measurement.GRAM, "Apple", nutritionTableTwo);
    private final static Recipe recipeOne = new Recipe("Valid recipe 1", Map.of(ingredientOne, 100, ingredientTwo, 200), "Description for this recipe", List.of("step one", "step two"), List.of(categoryOne), 15);
    private final static Recipe recipeTwo = new Recipe("Apfelkuchen", Map.of(ingredientOne, 250), "Mein liebster APfelkuchen", List.of("mjam mjam", "ich liebe kochen"), List.of(categoryTwo, categoryOne), 25);
    private final static Map<LocalDate, String> recipeMap1 = Map.of(LocalDate.now(), recipeOne.getUniqueCode(), LocalDate.now().plusDays(1), recipeTwo.getUniqueCode());
    private final static MealPlan planOne = new MealPlan(recipeMap1, recipeManager);
    private final static ShoppingList shoppingList = new ShoppingList(Map.of("r-192991", true, "r-89134", false), recipeManager);

    public static NutritionTable getValidNutritionTableOne() {
        return nutritionTableOne;
    }

    public static NutritionTable getValidNutritionTableTwo() {
        return nutritionTableTwo;
    }

    public static Ingredient getValidIngredientOne() {
        return ingredientOne;
    }

    public static Ingredient getValidIngredientTwo() {
        return ingredientTwo;
    }

    public static List<RecipeComponent> getValidIngredientList() {
        return List.of(ingredientOne, ingredientTwo);
    }

    public static Category getValidCategoryOne() {
        return categoryOne;
    }

    public static Category getValidCategoryTwo() {
        return categoryTwo;
    }

    public static List<Category> getValidCategoryList() {
        return List.of(categoryOne, categoryTwo);
    }

    public static Recipe getValidRecipeOne() {
        return recipeOne;
    }

    public static Recipe getValidRecipeTwo() {
        return recipeTwo;
    }

    public static MealPlan getValidMealPlanOne() {
        return planOne;
    }

    public static ShoppingList getShoppingList() {
        return shoppingList;
    }

    public static RecipeManager getRecipeManager() {
        return recipeManager;
    }
}
